// Copyright (C) 2008 Cory Strope <corystrope@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef INTREE_H_
#define INTREE_H_

#include <string>
#include <list>
#include <vector>
#include <exception>
#include "seqGenOptions.h"
#include "tree.h"
#include "treefile.h"
#include "aamodels.h"
#include "model.h"
#include "inClade.h"

using namespace std;

#define RANDOM 0
#define SINGLE_ROOT 1
#define MULTIPLE_ALIGNMENT_ROOT 2
#define DIST_CALC 0
#define TIME_SCALE 1
#define PREC_EPS 0.0001
#define ROOT_FLAG -2
#define SET 1
#define CHECK 2
#define PRECISION 7

// Forward definitions
class inClade;
class inMotif;

class inTree
{
public:
	// Variables
	bool randomInvariableAssignment;	
	bool scaleBranches, scaleTrees;
	char ma_columnCollapseMethod;
	double treeScale, branchScale;
	double proportion_invariable;
	double partitionRate;
	double scale;	
	int ma_numSeqsToUse;
	int rootSeqType;
	int partitionLength;
	int treeNum;
	short codon_offset;
	list<inMotif*> motif_specs;
	string ma_range;
	string filename;
	string invariable_by_partition;
	string rootseq_by_partition;
	vector<string> motifs_by_partition;
	string label;
	string tree;
	TTree *my_tree;
	string invariable;
	vector<string> input_MA;
	vector<string> input_MA_motifs;

	// Functions
	bool CheckPhylogeneticAncestralNodes(string& node_names, int action);
	double CalculatePathLengths();
	double SearchPath(TNode *anc, TNode *des, int flag);
	double SearchMinBranch(TNode *node, double curr_min);
	void Print_Trace_Header(stringstream& header, vector<string> otu_names, seqGenOptions* options);
	void Print_Branch(TNode *node, stringstream& header, vector<string>::iterator& taxon_names, int& num, seqGenOptions* options);
	double DoNudge(TNode *node, double step_size, int simulation_type);
	int  getRootSeq_and_Motif(int type);
	inMotif *FindMotif(char marker);
	inTree(const seqGenOptions *, int, inClade *);
	list <inMotif*> Read_MA(string filename);
	string addFilePath(string suffix, string prefix);
	void perform_checks(string, seqGenOptions *);
	void NudgeBranches(double step_size, int simulation_type);
	void ConvertRootSequence(TTree *tree);
	void TimeScale_Tree(double max_path, TNode *node);
	void BranchScale_Tree(TNode *anc, TNode *des, double scale);
	void Print_Time_Rel_Tree(ostream& out, bool writeAncestors);
	void PrintSubtree(TTree *tree, TNode *node, ostream& out, bool writeAncestors, int *nodeNo, bool time_rel);
	void Print_Tree(ostream& out);
	void Setup_Tree(seqGenOptions *options);
	void Define_Ancestors();
	void Define_Bipartitions(vector<string>& names);
	void Push_Bipartition(TNode *node, int which_tip);
	void Define_Anc(TNode *anc, TNode *des);
	void CPAN(TNode *node, string& node_names, int action, bool *inval);
	void SortTaxonTips(vector<string>& otu_names);
	void parseTree(string, seqGenOptions *);
	void parseLineages(seqGenOptions *options);
	void parseClade(string& clade_string, seqGenOptions *options);
	void parseMotif(string& motif_string, seqGenOptions *options);
	void parseIndel(string& indel_info, inClade *clade, seqGenOptions *options);
	void parsePound(string& subtree_info, inClade *clade, bool local, seqGenOptions *options);
	void propagateTreePointers();
	void propagateCladePointers(TNode *node);
	void setupTreeStructs(int partitionLength, seqGenOptions *options);
	void Reset_inMotif();
	void setMultiStateCharacters();

	private:
};

list<string> split(string str, string delim, bool mixDelim = false, bool merge = false);
string& trim(string& str);
string& Remove_Whitespace(string& str);

#endif /*INTREE_H_*/
