#ifndef _MOTIF_H_
#define _MOTIF_H_

#include <string>
#include <vector>
#include "tree.h"
#include "treefile.h"
#include <bitset>

using namespace std;

class siteProperties;
class varSite;
class inMotif;
class inClade;
class motifSite;
class Substitution;
class Insertion;
class Deletion;
class siteConstraints;
class Indel;
class TNode;

#define ACTIVE_MOTIF   4
#define DORMANT_MOTIF  5
#define VARIABLE_MOTIF 6

// For insertions. Left is the position to the left, right for right.
#define LEFT 0
#define RIGHT 1

// siteProperties general positions.
#define FIRST_POSITION 0
#define OTHER_POSITION 1
#define LAST_POSITION 2

#define MOTIF 0
#define TEMPLATE 1

class inMotif {
	public:
		string name;
		char marker;
		string regex;
		string sitemap;
		vector<bool> bipartition;
		bool sequence_template;

		inMotif();
		vector<siteProperties*> enumerateRegEx(TNode *node);
		bitset<20> getRegExValues(size_t which_site);
		bool isTemplate();
		void report();
};

class motifSite {
	public:
		list<siteProperties*> site_props;
		siteProperties *activeProps;
		TNode *my_node;

		motifSite(motifSite* site2copy = NULL);
		motifSite(motifSite *site_type, varSite *template_varSite, varSite *motif_varSite);

		void setActiveProps(vector<bool>& bipartition_to_test, motifSite *site, motifSite *anc, inClade *clade, bool last_site);
		void deleteMerge(motifSite *prev, motifSite *next);
		void deleteSiteObjects();
		void setNode(TNode *node);
		void copy(motifSite* site2copy);
		void init();
		void Print_Nulls();
};

class siteProperties {
	public:
	
		// Old stuff: Below. Making new stuff below. Eventually, new will replace old.
		Substitution *subst;
		Indel		 *indel;
		inMotif      *fromMotif[2];		// retain.
		bool		 ancestral;
		motifSite    *my_motifSite;
		
		siteProperties(vector<siteProperties*> site, varSite *var, inMotif *infromMotif, bool isTemplate);
		siteProperties(bool activeProp = false, motifSite *ms = NULL);
		siteProperties(motifSite *site_type, varSite *in_template_varSite, varSite *in_motif_varSite);

		void setMotifSite(motifSite *ms);
		void copy(siteProperties* stuff, short which, short type, motifSite *prev, siteProperties *anc, bool last_site);

		// OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD 
		siteProperties(siteProperties* site2copy, motifSite *anc);
		siteProperties(siteProperties* site2copy);
		void report();
};

class Substitution {
	public:
		bitset<20> substitution_bitstring;
		
		Substitution();
		Substitution(Substitution* site2copy);
		void copy(Substitution* site2copy);
		void setSiteBits(bitset<20> regex_site);
		string report_bitset();
		void setInvariable(char residue);
		bool siteInvariable();
};

class Indel {
	public:
		Insertion    *L_ins_, *R_ins_;	
		Deletion     *del;
		siteProperties *my_siteProperties;

		Indel(vector<siteProperties*> prev_site /*= NULL*/);
		Indel(Indel *site2copy, motifSite *anc, bool isTemplate);
		Indel(bool activeProp);
		Indel(varSite *in_template_varSite, varSite *in_motif_varSite);

		void setSiteProperties(siteProperties *sp);
		void setGeneral_varSites();
		void copy(Indel *site2copy, motifSite *prev, bool isTemplate, bool last_site, bool isDescendant, Indel *anc_copy);
		void report();
};

class Insertion {
	public:
		// Each one has a LEFT and RIGHT motif.
		varSite *my_sequence_template_varSite[2], *my_motif_varSite[2];
		varSite *on_site_motif_varSite, *on_site_sequence_template_varSite;	
		
		Insertion();
		Insertion(Insertion *site2copy, motifSite *anc, bool isTemplate, Insertion *anc_copy);
		Insertion(Deletion *site2copy, motifSite *anc, bool isTemplate);
		Insertion(varSite *in_template_varSite, varSite *in_motif_varSite);

		bool insertionAllowed(int size = 1);
		void copy(Insertion *site2copy, bool isTemplate, bool isDescendant, Insertion *anc_copy);
		void setMembership(varSite *mOS, bool isTemplate, size_t side);
};

class Deletion {
	public:
		varSite *my_sequence_template_varSite, *my_motif_varSite;

		Deletion();
		Deletion(Deletion *site2copy, motifSite *anc, bool isTemplate);
		Deletion(varSite *in_template_varSite, varSite *in_motif_varSite);

		bool deletionAllowed(int size = 1);
		void copy(Deletion *site2copy, bool isTemplate, bool isDescendant, Deletion *anc_copy);
		void setMembership(varSite *mOS, bool isTemplate);
};

class varSite {
	public:
		
		int min;
		int max;
		list<Deletion*> members;
		TNode *my_TNode;
		varSite *descendant_equiv;

		varSite(size_t min, size_t max, TNode *my_node);
		varSite(varSite *varSite2copy, TNode *my_node);

		bool insertion(int size = 1);
		bool deletion(int size = 1);
		void add2Member (Deletion *newMember);
		void removeFromMember (Deletion *thisMember);

		// OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD 
		inMotif* from_inMotif;
};

#endif
